/**
 * This program has been developed by a student from the bachelor Information Science at
 * Utrecht University within the Research Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

import React, { useState, useEffect, useMemo } from 'react';
import { NodeLinkViewModel } from './GeoNodeLinkViewModel';
import { useGraphQueryResult } from '../../data-access/store';
import { MapView, DeckGL } from 'deck.gl/typed';
import { getTooltip } from './Tooltip';
import { Node } from './types';
import * as d3 from 'd3';
import { generateBaseLayer } from './layers/BaseLayer';
import { FacetingLayer } from './layers/FacetingLayer';
import { ClusterLayer } from './layers/ClusterLayer';
import { NodeLinkLayer } from './layers/NodeLinkLayer';

type Props = {
  mapType: number;
  nodelink: 'cluster' | 'faceting' | 'nodelink';
  isAggregated: boolean;
  isFanned: boolean;
  brushing: boolean;
};

export const GeoNodeLinkMap = React.memo(({ mapType = 0, isAggregated, isFanned, nodelink = 'nodelink', brushing = false }: Props) => {
  const [graph, setGraph] = useState<NodeLinkViewModel | null>(null);
  const [isHovering, setIsHovering] = useState<boolean>();
  const [isLoading, setIsLoading] = useState<boolean>(true);
  const [viewport, setViewport] = useState<{ [key: string]: number }>({
    longitude: -102.4,
    latitude: 37.8,
    zoom: 3,
  });

  const graphQueryResult = useGraphQueryResult();

  useEffect(() => {
    if (graphQueryResult) {
      const graphModel = new NodeLinkViewModel({
        nodes: graphQueryResult.nodes as unknown as Node[], //TODO! must check if there is coordinates before accepting this cast
        edges: graphQueryResult.edges,
      });
      setGraph(graphModel);
      setIsLoading(false);
    }
  }, [graphQueryResult]);

  const onViewStateChange = (viewState: { [key: string]: number }) => {
    setViewport(viewState);
  };

  const edges = useMemo(() => graph?.getEdges(), [graph?.data]);
  const nodes = useMemo(() => graph?.getNodes(), [graph?.data]);

  const colorScale: d3.ScaleLinear<string, string> = d3
    .scaleLinear<string>()
    // .domain(d3.extent(edges, (d) => (d!.attributes!.prediction)))
    .domain([0, 10000]) // TODO: this is not ideal and makes the color mean nothing
    .range(['#ef8a62', '#1D6600', '#85327B']); // use theme?

  const layers: any = [generateBaseLayer(mapType)];

  if (nodelink === 'cluster') {
    layers.push(
      new ClusterLayer({
        graph: graph,
        edges: edges,
        nodes: nodes,
        colorScale: colorScale,
        brushing: brushing,
      })
    );
  } else if (nodelink === 'faceting') {
    layers.push(
      new FacetingLayer({
        graph: graph,
        edges: edges,
        nodes: nodes,
        colorScale: colorScale,
        isAggregated: isAggregated,
        isFanned: isFanned,
        brushing: brushing,
      })
    );
  } else {
    layers.push(
      new NodeLinkLayer({
        edges: edges,
        nodes: nodes,
        colorScale: colorScale,
        brushing: brushing,
      })
    );
  }

  return (
    <div>
      {!isLoading && (
        <DeckGL
          layers={layers}
          views={new MapView({ repeat: true, controller: true })}
          initialViewState={viewport}
          getTooltip={getTooltip}
          onViewStateChange={({ viewState }) => onViewStateChange(viewState)}
          onHover={({ object }) => setIsHovering(Boolean(object))}
          getCursor={({ isDragging }) => (isDragging ? 'grabbing' : isHovering ? 'pointer' : 'grab')}
        />
      )}
    </div>
  );
});

export default GeoNodeLinkMap;
